/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.se.eip;

import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.Set;

import org.ow2.petals.component.framework.se.AbstractServiceEngine;
import org.ow2.petals.se.eip.patterns.Pattern;

/**
 *
 * @author alouis TODO manage send sync and async in all patterns
 */
public class EIPComponent extends AbstractServiceEngine {

	private Map<String, Pattern> patterns;

	@Override
	public void doInit() {
		getLogger().info("Initialize the EIP Service Engine.");
		patterns = new HashMap<String, Pattern>();
		Properties properties = getComponentConfiguration().getProperties();
		Set<Object> patternsNames = properties.keySet();
		for (Object name : patternsNames) {
			try {
				String patternClassName = properties.getProperty(name
						.toString());
				getLogger().info(
						"  Loading the pattern " + name + " implemented by "
								+ patternClassName);

				Class<?> patternClass = this.getClass().getClassLoader()
						.loadClass(patternClassName);
				Pattern pattern = (Pattern) patternClass.newInstance();
				patterns.put(name.toString(), pattern);
				getLogger().info("  ...Pattern created.");
			} catch (ClassNotFoundException e) {
				getLogger().warning("  ...Class not found.");
			} catch (InstantiationException e) {
				getLogger().warning("  ...Pattern can not be instantiated.");
			} catch (IllegalAccessException e) {
				getLogger().warning(
						"  ...IllegalAccessException " + e.getMessage());
			} catch (ClassCastException e) {
				getLogger()
						.warning(
								"  ...The specified class does not implement the Pattern interface.");
			}
		}
	}

	public Map<String, Pattern> getPatterns() {
		return patterns;
	}
}
