/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */
package org.ow2.petals.se.eip.patterns;

import java.net.URI;
import java.util.List;

import javax.jbi.JBIException;
import javax.jbi.messaging.NormalizedMessage;

import org.ow2.petals.component.framework.Constants.MEPConstants;
import org.ow2.petals.component.framework.util.Exchange;
import org.ow2.petals.jbi.descriptor.Consumes;
import org.ow2.petals.se.eip.ExchangeContext;
/**
 *
 * @author alouis
 *
 */
public class RoutingSlip implements Pattern {

		/**
	 * Pipe all Consumes services defined in the service unit.<br> - If InOut,
	 * return the last result. <br> - If In, exchange is closed. <br> - If a
	 * Fault or Error occurs on a call, stop the process and return the Fault or
	 * Error
	 *
	 * @param exchange
	 */
	public void processPattern(Exchange exchange, ExchangeContext context) {

		context.getLogger().info("routing-slip");

		Exchange originalExchange = exchange;

		List<Consumes> consumesList = context.getSUConsumes(originalExchange
				.getEndpoint());
		// assumes all services to chain are based on the IN-OUT MEP
		// but the last call follow the MEP specified in the PROVIDES
		// section of the SU
		int count = consumesList.size();
		if (count > 0) {
			try {
				NormalizedMessage receivedMessage = originalExchange
						.getInMessage();
				for (int i = 0; i < count && receivedMessage != null; i++) {
					// For all calls, use InOut M.E.P.
					// For the last call, use the original M.E.P.
					URI mep = (i < count - 1) ? MEPConstants.IN_OUT_PATTERN
							.value() : originalExchange.getPattern();

					// create the exchange to pipe to the next service
					Exchange pipedExchange = PatternHelper.createExchange(
							consumesList.get(i), mep, receivedMessage, context);

					// send the exchange to the next service
					context.getLogger().info(
							"send exchange to ...\n - itf="
									+ consumesList.get(i).getInterfaceName()
									+ "\n - srv="
									+ consumesList.get(i).getServiceName()
									+ "\n - ep ="
									+ consumesList.get(i).getEndpointName());
					context.sendSync(pipedExchange);

					// process the response
					receivedMessage = PatternHelper.processSentExchange(pipedExchange,
							originalExchange, context);
				}
				// if the last piped service return a response, send it back
				// otherwise, the 'processSentExchange()' has set information
				// on the original exchange
				if (receivedMessage != null) {
					originalExchange.setOutMessage(receivedMessage);
				}
			} catch (JBIException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
	}

}
